#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from w3bsite.classes.config import *
from w3bsite.classes import utils, views
from w3bsite.classes import defaults as _defaults_

# the payments requests.
class Requests(_defaults_.Defaults):
	def __init__(self,
		# passed Website.x objects.
		stripe=None,
		defaults=None,
	):
		# defaults.
		_defaults_.Defaults.__init__(self)
		defaults.stripe = stripe
		self.assign(defaults.dict())
		
		# urlpatterns.
		self.urls = views.build_urls([
			self.subscriptions.Purchase(defaults=defaults),
			self.subscriptions.Create(defaults=defaults),
			self.subscriptions.Delete(defaults=defaults),
			self.subscriptions.List(defaults=defaults),
			self.methods.Create(defaults=defaults),
			self.methods.Delete(defaults=defaults),
			self.methods.List(defaults=defaults),
		])

	# the subscriptions.
	class subscriptions:

		# all-in-one purchase request.
		class Purchase(views.Request):
			def __init__(self, defaults=None,):
				_defaults_.Defaults.__init__(self)
				self.assign(defaults.dict())
				views.Request.__init__(self, "requests/payments/subscriptions/", "purchase")
			def request(self, request):

				# check overall rate limit.
				response = self.rate_limit.verify(ip=utils.get_client_ip(request), mode="daily", limit=1000, reset_minutes=3600*24, increment=True)
				if not response.success: return self.response(response)

				# check authenticated.
				response = self.users.authenticated(request)
				if not response.success: return self.response(response)
				email = response.email
				try: api_key = response.api_key
				except: api_key = None

				# get parameters.
				parameters, response = self.parameters.get(request, [
					"product",
					"plan",
					"card_name",
					"card_number",
					"card_expiration_month",
					"card_expiration_year",
					"card_cvc",])
				if not response.success: return self.response(response)

				# request.
				return self.response(self.users.create_subscription(
					email=email,
					api_key=api_key,
					product=parameters["product"],
					plan=parameters["plan"],
					card_name=parameters["card_name"],
					card_number=parameters["card_number"],
					card_cvc=parameters["card_cvc"],
					card_expiration_month=parameters["card_expiration_month"],
					card_expiration_year=parameters["card_expiration_year"],))

		# create subscription.
		class Create(views.Request):
			def __init__(self, defaults=None,):
				_defaults_.Defaults.__init__(self)
				self.assign(defaults.dict())
				views.Request.__init__(self, "requests/payments/subscriptions/", "create")
			def request(self, request):

				# check overall rate limit.
				response = self.rate_limit.verify(ip=utils.get_client_ip(request), mode="daily", limit=1000, reset_minutes=3600*24, increment=True)
				if not response.success: return self.response(response)

				# check authenticated.
				response = self.users.authenticated(request)
				if not response.success: return self.response(response)
				email = response.email
				try: api_key = response.api_key
				except: api_key = None

				# get parameters.
				parameters, response = self.parameters.get(request, [
					"product",
					"plan",])
				if not response.success: return self.response(response)

				# request.
				return self.response(self.users.create_subscription(
					email=email,
					api_key=api_key,
					product=parameters["product"],
					plan=parameters["plan"],
					))

		# delete subscription.
		class Delete(views.Request):
			def __init__(self, defaults=None,):
				_defaults_.Defaults.__init__(self)
				self.assign(defaults.dict())
				views.Request.__init__(self, "requests/payments/subscriptions/", "delete")
			def request(self, request):

				# check overall rate limit.
				response = self.rate_limit.verify(ip=utils.get_client_ip(request), mode="daily", limit=1000, reset_minutes=3600*24, increment=True)
				if not response.success: return self.response(response)

				# check authenticated.
				response = self.users.authenticated(request)
				if not response.success: return self.response(response)
				email = response.email

				###################
				# by subscription id.

				# get parameters.
				parameters, response = self.parameters.get(request, [
					"subscription_id",])
				if response.success: 

					# request.
					return self.response(self.stripe.subscriptions.cancel(
						email=email,
						subscription_id=parameters["subscription_id"],))

				###################
				# by plan & product.
				else:

					# get parameters.
					parameters, response = self.parameters.get(request, [
						"product",
						"plan",])
					if not response.success: return self.response(response)

					# convert plan.
					response = self.stripe.get_plan_id(product=parameters["product"], plan=parameters["plan"])
					if not response.success: return self.response(response)
					plan_id = response.id

					# request.
					return self.response(self.stripe.subscriptions.cancel(
						email=email,
						plan=plan_id,))

		# list methods.
		class List(views.Request):
			def __init__(self, defaults=None,):
				_defaults_.Defaults.__init__(self)
				self.assign(defaults.dict())
				views.Request.__init__(self, "requests/payments/subscriptions/", "list")
			def request(self, request):

				# check overall rate limit.
				response = self.rate_limit.verify(ip=utils.get_client_ip(request), mode="daily", limit=1000, reset_minutes=3600*24, increment=True)
				if not response.success: return self.response(response)

				# check authenticated.
				response = self.users.authenticated(request)
				if not response.success: return self.response(response)
				email = response.email

				# get customer id.
				response = self.stripe.customers.get_id(email=email)
				if not response.success: return self.response(response)
				customer_id = response.id

				# list.
				response = self.stripe.subscriptions.get(email=email, active_only=True)
				if not response.success: return self.response(response)
				subscriptions = response.subscriptions
				for plan_id in list(subscriptions.keys()):
					response = self.stripe.get_product_id_by_plan_id(plan_id)
					if not response.success: return self.response(response)
					product_id = response.id
					response = self.stripe.get_product_name(product_id)
					if not response.success: return self.response(response)
					product = response.name
					response = self.stripe.get_plan_name(plan_id)
					if not response.success: return self.response(response)
					plan = response.name
					subscriptions[plan_id]["product"] = product
					subscriptions[plan_id]["plan"] = plan
					subscriptions[plan_id]["favicon"] = self.stripe.template_data["PRODUCTS"][product][plan]["favicon"]
					subscriptions[plan_id]["price"] = self.stripe.template_data["PRODUCTS"][product][plan]["price"]

				# request.
				return self.success(f"Successfully listed {len(subscriptions)} active subscription(s).", {
					"subscriptions":subscriptions,
				})

	# the payment methods.
	class methods:

		# create method.
		class Create(views.Request):
			def __init__(self, defaults=None,):
				_defaults_.Defaults.__init__(self)
				self.assign(defaults.dict())
				views.Request.__init__(self, "requests/payments/methods/", "create")
			def request(self, request):

				# check overall rate limit.
				response = self.rate_limit.verify(ip=utils.get_client_ip(request), mode="daily", limit=1000, reset_minutes=3600*24, increment=True)
				if not response.success: return self.response(response)

				# check authenticated.
				response = self.users.authenticated(request)
				if not response.success: return self.response(response)
				email = response.email

				# get parameters.
				parameters, response = self.parameters.get(request, [
					"number",
					"month",
					"year",
					"cvc",])
				if not response.success: return self.response(response)

				# get customer id.
				response = self.stripe.customers.get_id(email=email)
				if not response.success: return self.response(response)
				customer_id = response.id

				# request.
				return self.response(self.stripe.customers.create_card(
					id=customer_id,
					number=parameters["number"],
					month=parameters["month"],
					year=parameters["year"],
					cvc=parameters["cvc"],))

		# delete method.
		class Delete(views.Request):
			def __init__(self, defaults=None,):
				_defaults_.Defaults.__init__(self)
				self.assign(defaults.dict())
				views.Request.__init__(self, "requests/payments/methods/", "delete")
			def request(self, request):

				# check overall rate limit.
				response = self.rate_limit.verify(ip=utils.get_client_ip(request), mode="daily", limit=1000, reset_minutes=3600*24, increment=True)
				if not response.success: return self.response(response)

				# check authenticated.
				response = self.users.authenticated(request)
				if not response.success: return self.response(response)
				email = response.email

				# get customer id.
				response = self.stripe.customers.get_id(email=email)
				if not response.success: return self.response(response)
				customer_id = response.id

				# request.
				return self.response(self.stripe.customers.delete_card(
					id=customer_id,
					))

		# list methods.
		class List(views.Request):
			def __init__(self, defaults=None,):
				_defaults_.Defaults.__init__(self)
				self.assign(defaults.dict())
				views.Request.__init__(self, "requests/payments/methods/", "list")
			def request(self, request):

				# check overall rate limit.
				response = self.rate_limit.verify(ip=utils.get_client_ip(request), mode="daily", limit=1000, reset_minutes=3600*24, increment=True)
				if not response.success: return self.response(response)

				# check authenticated.
				response = self.users.authenticated(request)
				if not response.success: return self.response(response)
				email = response.email

				# get customer id.
				response = self.stripe.customers.get_id(email=email)
				if not response.success: return self.response(response)
				customer_id = response.id

				# request.
				return self.response(self.stripe.customers.get_cards(
					id=customer_id,))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from w3bsite.classes.config import *

# the users class.
class Logging(object):
	def __init__(self, name=None, root=None, database=None):

		# docs.
		DOCS = {
			"module":"website.logging", 
			"initialized":True,
			"description":[], }

		# attributes.
		self.name = name
		self.root = root
		self.database = database
		self.log_file = f"{self.root}/logs/logs"
		#if not Files.exists(f"{self.database}/logs"):
		#	Files.create(f"{self.database}/logs", directory=True)

	# log to logfile.
	def log(self,
		# option 1.
		# 	the message response body.
		message=None,
		# option 2.
		# 	the error response body.
		error=None,
		# option 3.
		# 	the entire response.
		response=None,
	):
		# get options.
		msg = None
		if message != None:
			msg = message
		elif error != None:
			msg = "Error: "+error
		elif response != None:
			if response.success:
				msg = response["message"]
			else:
				msg = "Error: "+response.error

		# log to file.
		__log__(msg)

		#

	# system functions.
	def __log__(self, string):
		msg = f"{Date().seconds_timestamp}: {string}"
		print(msg)
		with open(self.log_file, "a") as file:
			file.write(msg)

	#


"""
Django settings for website project.

Generated by 'django-admin startproject' using Django 3.0.8.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import os
from dev0s.shortcuts import *

# dev0s.env.
SOURCE = Directory(gfp.base(__file__, back=3))
DATABASE = Directory(dev0s.env.get("DATABASE", default=str(SOURCE)))
PRODUCTION = dev0s.env.get("PRODUCTION", default=True, format=bool)
DOMAIN = dev0s.env.get("DOMAIN", default=None)
SECRET_KEY = dev0s.env.get("DJANGO_SECRET_KEY", default=String().generate(length=128, capitalize=True, digits=True, special=True))
#WEBSITE_BASE = dev0s.env.get("WEBSITE_BASE", default=None)
#if WEBSITE_BASE == None:
#    raise ValueError("Improperly configured run, env variable [WEBSITE_BASE] is None.")
    
# Match production,
if PRODUCTION:

    # disable logs.
    DEBUG = False

    # security settings.
    SECURE_SSL_REDIRECT = True
    CSRF_COOKIE_SECURE = True
    SESSION_COOKIE_SECURE = True
    SESSION_COOKIE_HTTPONLY = True
    SECURE_BROWSER_XSS_FILTER = True
    SECURE_CONTENT_TYPE_NOSNIFF = True
    SECURE_BROWSER_XSS_FILTER = True
    SECURE_HSTS_PRELOAD = True
    X_FRAME_OPTIONS = 'DENY'

else:
    SECURE_SSL_REDIRECT = False
    DEBUG = True


# Add allowed hosts.
ALLOWED_HOSTS = []
if DOMAIN not in [None, ""]:
    for url in [
        DOMAIN,
    ]:
        if url != None:
            ALLOWED_HOSTS.append(url)
            ALLOWED_HOSTS.append("www."+url)
            ALLOWED_HOSTS.append("http://"+url)
            ALLOWED_HOSTS.append("https://"+"www."+url)
if PRODUCTION == False:
    ALLOWED_HOSTS.append("*")

# Application definition
INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = '__defaults__.django.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            SOURCE.join("apps"),
            str(dev0s.defaults.vars.site_packages),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = '__defaults__.django.wsgi.application'


# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': DATABASE.join("data/db.sqlite3"),
    }
}


# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_ROOT = f"/www-data/static/"
STATIC_URL = '/static/'
STATICFILES_DIRS = [
    SOURCE.join('__defaults__/static'),
]
